import { createCallback, createSubRoot } from "@solid-primitives/rootless";
import {
  createRenderEffect,
  createSignal,
  Index,
  Match,
  onCleanup,
  onMount,
  Switch,
  type Component,
} from "solid-js";
import { render } from "solid-js/web";
import Scaffold from "~material/Scaffold";
import { isPointNotInRect } from "./dom";
import "./MediaQuickview.css";
import AppTopBar from "~material/AppTopBar";
import { IconButton } from "@suid/material";
import { Close } from "@suid/icons-material";

function renderIsolateMediaQuickview(
  each: QuickviewMedia[],
  index: number,
  transitionFrom?: Element,
) {
  createSubRoot((disposeAll) => {
    let container: HTMLDivElement;

    createRenderEffect(() => {
      container = document.createElement("div");
      container.setAttribute("role", "presentation");
      container.classList.add("MediaQuickview__root");
      document.querySelector("body")!.appendChild(container);

      onCleanup(() => container.remove());

      const dispose = render(() => {
        return (
          <MediaQuickview
            each={each}
            defaultIndex={index}
            transitonFrom={transitionFrom}
            onClose={disposeAll}
          />
        );
      }, container);

      onCleanup(dispose);
    });
  });
}

export function createMediaQuickview() {
  return createCallback(renderIsolateMediaQuickview);
}

function ImagePage(props: { src: string; alt?: string }) {
  const [scale, setScale] = createSignal(1);
  const [offsetX, setOffsetX] = createSignal(0);
  const [offsetY, setOffsetY] = createSignal(0);

  const onWheel = (event: WheelEvent & { currentTarget: HTMLElement }) => {
    // This is a de-facto standard for scaling:
    // Browsers will simulate ctrl + wheel for two point scaling gesture.
    if (event.ctrlKey) {
      event.preventDefault();
      event.stopPropagation();

      const offset = event.deltaY;

      setScale((x) => x + offset / event.currentTarget.clientHeight);
    }
  };

  return (
    <img
      src={props.src}
      alt={props.alt}
      onWheel={onWheel}
      style={{
        transform: `scale(${scale()}) translateX(${offsetX()}) translateY(${offsetY()})`,
      }}
      onLoad={({ currentTarget }) => {
        const { top, left, width, height } =
          currentTarget.getBoundingClientRect();
      }}
    ></img>
  );
}

export type QuickviewMedia = {
  cat: "image" | "video" | "gifv" | "audio" | "unknown";
  src: string;
  alt?: string;
};

export type MediaQuickviewProps = {
  each: QuickviewMedia[];
  defaultIndex: number;
  transitonFrom?: Element;

  onClose?(): void;
};

const MediaQuickview: Component<MediaQuickviewProps> = (props) => {
  let root: HTMLDialogElement;
  const [lightOut, setLightOut] = createSignal(false);

  function onDialogClick(
    event: MouseEvent & { currentTarget: HTMLDialogElement },
  ) {
    event.stopPropagation();

    if (
      isPointNotInRect(
        event.currentTarget.getBoundingClientRect(),
        event.clientX,
        event.clientY,
      )
    ) {
      event.currentTarget.close();
    } else {
      setLightOut((x) => !x);
    }
  }

  return (
    <dialog
      ref={(e) => {
        root = e;
        onMount(() => {
          e.showModal();
        });
      }}
      class="MediaQuickview"
      classList={{ lightout: lightOut() }}
      onClose={props.onClose}
      onCancel={props.onClose}
      onClick={onDialogClick}
    >
      <Scaffold
        topbar={
          <AppTopBar>
            <IconButton color="inherit" onClick={(e) => root.close()}>
              <Close />
            </IconButton>
          </AppTopBar>
        }
      >
        <div
          ref={(e) => {
            onMount(() => {
              e.children.item(props.defaultIndex)!.scrollIntoView({
                behavior: "instant",
                inline: "center",
              });
            });
          }}
          class="pages"
        >
          <Index each={props.each}>
            {(item, index) => {
              return (
                <section class="page">
                  <Switch>
                    <Match when={item().cat === "image"}>
                      <ImagePage src={item().src} alt={item().alt} />
                    </Match>
                  </Switch>
                </section>
              );
            }}
          </Index>
        </div>
      </Scaffold>
    </dialog>
  );
};

export default MediaQuickview;
